////////////////////////////////////////////////////////////
//Test SL015M-3
//MCS51 source code
////////////////////////////////////////////////////////////       
#include <reg52.h>
#include <string.h>


#define  OSC_FREQ        22118400L
#define  TIME0_10ms      65536L - OSC_FREQ/1200L

#define  BOUND4800       256 - 24
#define  BOUND9600       256 - 12
#define  BOUND14400      256 - 8
#define  BOUND19200      256 - 6
#define  BOUND28800      256 - 4
#define  BOUND38400      256 - 3
#define  BOUND57600      256 - 2
#define  BOUND115200     256 - 1


sbit     CARDIN    =    P1^0;                                                      

//Command List, preamble + length + command code + data frame
unsigned char code GetCardInformation[3]      =  {0xBA,0x02,0x31};       
unsigned char code GetBlcok0SecurityStatus[5] =  {0xBA,0x04,0x32,0x0,0x01};
unsigned char code ReadBlocks[5]              =  {0xBA,0x04,0x33,0x0,0x10};                                     
unsigned char code WriteBlock0[8]             =  {0xBA,0x07,0x34,0x0,0x11,0x22,0x33,0x44};                                
unsigned char code WriteAFI[4]                =  {0xBA,0x03,0x35,0xAA};  
unsigned char code WriteDSFID[4]              =  {0xBA,0x03,0x36,0xBB};                                
unsigned char code LockBlock0[4]              =  {0xBA,0x03,0x37,0x0};                                
unsigned char code LockAFI[3]                 =  {0xBA,0x02,0x38};                                          
unsigned char code LockDSFID[3]               =  {0xBA,0x02,0x39};    
unsigned char code TrunOnRed[4]               =  {0xBA,0x03,0x40,0x01}; 
unsigned char code TrunOffRed[4]              =  {0xBA,0x03,0x40,0x0};              

void InitializeSystem();                                           
void SendCom(unsigned char *g_cCommand);                           
void StartTime(unsigned int _MS);                                  
void StopTime();
                           
unsigned char g_cReceBuf[70];                                      
unsigned char g_cReceNum;                                          
unsigned char g_cWatchTime;                                        

bit   g_bReceOver;                                                 
bit   g_bReceReady;
bit   g_bTimeOut;                                                  

void main() 
{
     InitializeSystem();
     
     while(1)
     {
         while (!CARDIN)
         {

             SendCom(GetCardInformation);                      
             StartTime(2);                                     //
             while (!g_bReceOver && !g_bTimeOut);              //waitting until receive response from SL013 or over 20ms            
             StopTime();                                       //
             if ((g_cReceBuf[2]!=0x31) || (g_cReceBuf[3]!=0) || g_bTimeOut)
             {   break;   }                                    
 
             SendCom(GetBlcok0SecurityStatus);                
             StartTime(2);
             while (!g_bReceOver && !g_bTimeOut);
             StopTime();
             if ((g_cReceBuf[2]!=0x32) || (g_cReceBuf[3]!=0) || g_bTimeOut)
             {    break;    }     
             
             if (g_cReceBuf[4] != 0)
             {    break;    } 
             
             SendCom(WriteBlock0);                 
             StartTime(2);
             while (!g_bReceOver && !g_bTimeOut);
             StopTime();
             if ((g_cReceBuf[2]!=0x34) || (g_cReceBuf[3]!=0) || g_bTimeOut)
             {    break;    }
             if (memcmp(&WriteBlock0[4], &g_cReceBuf[4], 4) != 0)
             {    break;    }                 
             
             SendCom(ReadBlocks);                 
             StartTime(3);
             while (!g_bReceOver && !g_bTimeOut);
             StopTime();
             if ((g_cReceBuf[2]!=0x33) ||(g_cReceBuf[3]!=0) || g_bTimeOut)
             {    break;    } 
             if (memcmp(&WriteBlock0[4], &g_cReceBuf[4], 4) != 0)
             {    break;    }                   

             SendCom(WriteAFI);                
             StartTime(2);
             while (!g_bReceOver && !g_bTimeOut);
             StopTime();
             if ((g_cReceBuf[2]!=0x35) || (g_cReceBuf[3]!=0) || g_bTimeOut)
             {    break;    } 
             if (memcmp(&WriteAFI[3], &g_cReceBuf[4], 1) != 0)
             {    break;    } 
                          
             SendCom(WriteDSFID);                 
             StartTime(2);
             while (!g_bReceOver && !g_bTimeOut);
             StopTime();
             if ((g_cReceBuf[2]!=0x36) || (g_cReceBuf[3]!=0) || g_bTimeOut)
             {    break;    } 
             if (memcmp(&WriteDSFID[3], &g_cReceBuf[4], 1) != 0)
             {    break;    }     

/*            
             SendCom(LockBlock0);                 
             StartTime(8);
             while (!g_bReceOver && !g_bTimeOut);
             StopTime();
             if ((g_cReceBuf[2]!=0x37) || (g_cReceBuf[3]!=0) || g_bTimeOut)
             {    break;    }

             SendCom(LockAFI);                 
             StartTime(8);
             while (!g_bReceOver && !g_bTimeOut);
             StopTime();
             if ((g_cReceBuf[2]!=0x38) || (g_cReceBuf[3]!=0) || g_bTimeOut)
             {    break;    }    
             
             SendCom(LockDSFID);                 
             StartTime(8);
             while (!g_bReceOver && !g_bTimeOut);
             StopTime();
             if ((g_cReceBuf[2]!=0x39) || (g_cReceBuf[3]!=0) || g_bTimeOut)
             {    break;    }                           
*/
    
             SendCom(TrunOnRed);                
             StartTime(50);
             while (!g_bTimeOut);
             SendCom(TrunOffRed);                
             StartTime(50);
             while (!g_bTimeOut);
         }
     }             
       
}

        
////////////////////////////////////////////////////////////
void InitializeSystem()
{
    TMOD &= 0x0F; 
    TMOD |= 0x21; 
    PCON |= 0x80;
    SCON  = 0x50; 
    TH1 = TL1 = BOUND115200; 
    g_cReceNum = 0;
    g_bReceOver = 0;
    g_bReceReady = 0;
    TR1   = 1;
    IE   |= 0x90;
}

////////////////////////////////////////////////////////////
void StartTime(unsigned int _MS)
{
    TH0 = (unsigned char)((TIME0_10ms>>8)&0xFF);
    TL0 = (unsigned char)(TIME0_10ms&0xFF);
    g_cWatchTime = _MS;
    g_bTimeOut = 0;
    ET0 = 1;
    TR0 = 1;
}
	
////////////////////////////////////////////////////////////
time0_int () interrupt 1 using 1
{
    TH0 = (unsigned char)((TIME0_10ms>>8) & 0xFF);
    TL0 = (unsigned char)(TIME0_10ms & 0xFF);
    if (g_cWatchTime--==0)
    {    g_bTimeOut = 1;    }
}

////////////////////////////////////////////////////////////
void StopTime()
{
    ET0 = 0;
    TR0 = 0;
}	

////////////////////////////////////////////////////////////
//Send command to SL015M-3
////////////////////////////////////////////////////////////
void SendCom(unsigned char *g_cCommand)
{    
     unsigned char i,chkdata,sendleg;

     ES = 0;
     sendleg = *(g_cCommand+1) + 1;
     
     chkdata = 0;
     for(i=0; i<sendleg; i++)
     {    
     	  chkdata ^= *(g_cCommand+i);
     	  
     	  TI = 0;
     	  SBUF = *(g_cCommand+i);
     	  while(!TI);
     }
     
     TI = 0;
     SBUF = chkdata;
     while (!TI);
     TI = 0;
     
     g_bReceReady = 1;
     g_bReceOver = 0;
     ES = 1;
}

/////////////////////////////////////////////////////////////////////
//Get result from SL015M-3
/////////////////////////////////////////////////////////////////////
seri_int () interrupt 4 using 1
{
   unsigned char i,j;
   unsigned char rece_data = SBUF;
   unsigned char verify = 0;
   
   if(RI)
   {    
       RI = 0;
       if (g_bReceReady)
       {  
           switch (rece_data)
           {    
               case 0xBD:
                    g_bReceReady = 0;
                    g_bReceOver = 0;
                    g_cReceNum = 0;
                    g_cReceBuf[0] = 0xBD;
                    g_cReceBuf[1] = 0;
                    break;
               default:
                    g_cReceNum++;
                    break;
           }
       }
       else
       {    
           i = ++g_cReceNum;
           g_cReceBuf[i] = rece_data;
	   if (g_cReceBuf[1] == g_cReceNum - 1)
	   {
	       for (j=0; j<g_cReceNum+1; j++)
               {   verify ^= g_cReceBuf[j];    }
               g_bReceOver = 1;
               ES = 0;
           }
           if (g_cReceNum > sizeof(g_cReceBuf))
           {
               g_bReceOver = 0;
               g_cReceNum = 0;
           }
        }
    }
    
    if(TI)
    {   
        TI = 0; 
    }
}
